package gravity;

import java.io.EOFException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.LinkedList;

public class FileInputHandler implements InputHandler {
	
	private final int queueSize = 10;
	
	private volatile LinkedList<ByteBuffer> queue = new LinkedList<>();
	private Object loadWaitObject = new Object();
	private boolean endOfFile = false;
	
	private RandomAccessFile saveFile;
	
	public final Thread thread;
	
	public FileInputHandler(File file) throws FileNotFoundException {
		saveFile = new RandomAccessFile(file, "r");
		thread = new Thread(this::performLoads);
	}
	
	public void startReading() {
		if (!thread.isAlive()) thread.start();
	}
	
	public ByteBuffer getBuffer() {
		if (queue.isEmpty()) {
			if (endOfFile) {
				Simulation.simulation.running = false;
				System.out.println("Reached the end of the file.");
			}
			return null;
		}
		
		ByteBuffer b = queue.removeFirst();
		synchronized (loadWaitObject) {
			loadWaitObject.notify();
		}
		return b;
	}
	
	private void performLoads() {
		while (!Thread.interrupted()) {
			if (queue.size() == queueSize) {
				try {
					synchronized (loadWaitObject) {
						loadWaitObject.wait();
					}
				} catch (InterruptedException e) {
					break;
				}
			}
			ByteBuffer buffer;
			try {
				int count = saveFile.readInt();
				buffer = ByteBuffer.allocate(count);
				saveFile.getChannel().read(buffer);
				buffer.flip();
			} catch (EOFException eof) {
				endOfFile = true;
				break;
			} catch (IOException e) {
				e.printStackTrace();
				break; // TODO: Might change to continue
			}
			queue.addLast(buffer);
		}
	}
	
	public void skipFrames(int frames) {}
	
	public void destroy() {
		thread.interrupt();
	}
	
	public ByteBuffer getMetaData() throws IOException {
		FileChannel fc = saveFile.getChannel();
		saveFile.seek(0);
		
		if (saveFile.readByte() != 'G') {
			saveFile.seek(0);
			throw new IOException("This is not a Gravity savefile");
		}
		
		byte version = saveFile.readByte();
		if (version != 0x01) System.err.println("The version of this file (" + Integer.toHexString(version) + ") is not supported.\nAttempting to read it using version 01");
		int totalFrames = saveFile.readInt();
		short metaLen = saveFile.readShort();
		
		ByteBuffer meta = ByteBuffer.allocate(metaLen + 4);
		meta.putInt(totalFrames);
		fc.read(meta);
		meta.flip();
		
		return meta;
	}
	
}
