package gravity;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.util.LinkedList;

public class FileOutputHandler implements OutputHandler {
	
	private volatile LinkedList<ByteBuffer> saveQueue = new LinkedList<>();
	private Object saveWaitObject = new Object();
	
	private RandomAccessFile saveFile;
	
	public final Thread thread;
	
	public FileOutputHandler(File file) throws FileNotFoundException {
		saveFile = new RandomAccessFile(file, "rw");
		
		thread = new Thread(this::performSaves);
		thread.start();
	}
	
	public void pushBuffer(ByteBuffer buffer) {
		saveQueue.addLast(buffer);
		synchronized (saveWaitObject) {
			saveWaitObject.notify();
		}
	}
	
	private void performSaves() {
		int totalFrames = 0;
		while (true) {
			if (saveQueue.isEmpty()) {
				if (Thread.interrupted()) break;
				try {
					synchronized (saveWaitObject) {
						saveWaitObject.wait();
					}
				} catch (InterruptedException e) {
					break;
				}
			}
			ByteBuffer buffer = saveQueue.removeFirst();
			try {
				saveFile.getChannel().write(buffer);
				long prev = saveFile.getFilePointer();
				saveFile.seek(2);
				saveFile.writeInt(++totalFrames);
				saveFile.seek(prev);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	public void destroy() {
		thread.interrupt();
	}
	
	public void writeMetaData(byte[] meta) throws IOException {
		saveFile.seek(0);
		saveFile.writeByte('G');
		saveFile.writeByte(0x01);
		saveFile.writeInt(0);
		saveFile.writeShort(meta.length);
		saveFile.write(meta);
	}
}
